<?php

namespace EventPortal\Providers;

use EventPortal\Commands\SeedAll;
use EventPortal\Commands\SeedPermissionsAndRoles;
use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\Venue\Models\Venue;
use EventPortal\Exceptions\Handler;
use EventPortal\Http\Middleware\CheckPermission;
use EventPortal\Http\Middleware\CurrentTeam;
use Illuminate\Contracts\Debug\ExceptionHandler;
use Illuminate\Routing\Router;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\ServiceProvider;
use Inertia\Inertia;

class AppServiceProvider extends ServiceProvider
{
    /** @var string */
    protected $root = __DIR__ . '/../..';

    public function boot(Router $router): void
    {
        $this->app->booted(function (): void {
            $this->loadRoutesFrom("{$this->root}/routes/routes.php");
        });

        $this->loadViewsFrom("{$this->root}/resources/views", 'EventPortal');

        $this->loadMigrationsFrom("{$this->root}/database/migrations");

        $this->loadFactoriesFrom("{$this->root}/database/factories");

        $this->app->bind(ExceptionHandler::class, Handler::class);

        $this->publishes([
            "{$this->root}/tables/delegate.php" => '/tables/delegates.php',
            "{$this->root}/tables/venue.php" => '/tables/venue.php',
            "{$this->root}/tables/event.php" => '/tables/event.php',
            "{$this->root}/tables/event-delegate.php" => '/tables/event-delegate.php',
        ], 'event-portal-tables');

        $this->publishes([
            "{$this->root}/forms/delegate.php" => '/forms/delegates.php',
            "{$this->root}/forms/event.php" => '/forms/event.php',
            "{$this->root}/forms/venue.php" => '/forms/venue.php',
            "{$this->root}/forms/event-delegate.php" => '/forms/event-delegate.php',
        ], 'event-portal-forms');

        $this->publishes([
            "{$this->root}/config/permission.php" => config_path('/permission.php'),
            "{$this->root}/config/state-machine.php" => config_path('/state-machine.php'),
            "{$this->root}/config/audit.php" => config_path('/audit.php'),
            "{$this->root}/config/page-headers.php" => config_path('/page-headers.php'),
        ], 'event-portal-config');

        if ($this->app->runningInConsole()) {
            $this->commands([
                SeedAll::class,
                SeedPermissionsAndRoles::class,
            ]);
        }

        $this->app->bind(\Illuminate\Contracts\Debug\ExceptionHandler::class, \EventPortal\Exceptions\Handler::class);

        $router->aliasMiddleware('current-team', CurrentTeam::class);
        $router->aliasMiddleware('check-permission', CheckPermission::class);

        Inertia::setRootView('EventPortal::app');

        Inertia::share([
            'errors' => function () {
                return session('errors') ? session('errors')->getBag('default')->getMessages() : (object) [];
            },
            'user' => function () {
                return Auth::user() ? [
                    'id' => Auth::user()->id,
                    'permissions' => optional(Auth::user()->admin)->allPermissions,
                    'roles' => optional(Auth::user()->admin)->allRoles,
                ] : null;
            },
        ]);

        Route::model('event', Event::class);
        Route::model('delegate', Delegate::class);
        Route::model('venue', Venue::class);
        Route::model('eventDelegate', EventDelegate::class);
        Route::model('admin', Admin::class);
    }
}
